# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from openerp.tools.translate import _

class account_move(osv.osv):
    _name = 'account.move'
    _inherit = ['account.move','mail.thread']

    def post(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice = context.get('invoice', False)
        valid_moves = self.validate(cr, uid, ids, context)

        if not valid_moves:
            raise osv.except_osv(_('Error!'), _('You cannot validate a non-balanced entry.\nMake sure you have configured payment terms properly.\nThe latest payment term line should be of the "Balance" type.'))
        obj_sequence = self.pool.get('ir.sequence')
        for move in self.browse(cr, uid, valid_moves, context=context):
            if move.name =='/':
                new_name = False
                journal = move.journal_id

                if invoice and invoice.internal_number:
                    new_name = invoice.internal_number
                else:
                    if journal.sequence_id:
                        # c = {'fiscalyear_id': move.period_id.fiscalyear_id.id}
                        c = {'source_date' : move.date}
                        new_name = obj_sequence.next_by_id(cr, uid, journal.sequence_id.id, c)
                    else:
                        raise osv.except_osv(_('Error!'), _('Please define a sequence on the journal.'))

                if new_name:
                    self.write(cr, uid, [move.id], {'name':new_name})

        cr.execute('UPDATE account_move '\
                   'SET state=%s '\
                   'WHERE id IN %s',
                   ('posted', tuple(valid_moves),))
        return True
    def _selection_origin_oe_id(self, cr, uid, context=None):
        obj_model = self.pool.get('ir.model')

        ids = obj_model.search(cr, uid, [])
        model = obj_model.read(cr, uid, ids, ['model','name'])

        return [(m['model'], m['name']) for m in model] 

    def _function_origin_document(self, cr, uid, ids, field_name, args, context=None):
        if context is None:
            context = {}

        res = {}
        obj_model = self.pool.get('ir.model')

        for move in self.browse(cr, uid, ids):
            res[move.id] =  {
                            'function_origin_doc' : '',
                            'origin_model_id' : False,
                            'origin_res_id' : 0,
                            }
            if move.origin_type == 'oe_doc':
                if move.origin_oe_doc_id:
                    criteria = [('model','=',move.origin_oe_doc_id._name)]
                    model_ids = obj_model.search(cr, uid, criteria)
                    if model_ids:
                        res[move.id]['origin_model_id'] = model_ids[0]
                        res[move.id]['origin_res_id'] = move.origin_oe_doc_id.id
                    
        return res

    def _default_origin_type(self, cr, uid, context={}):
        return 'manual'

    _columns =  {
                'origin_type' : fields.selection(string='Origin Type', 
                    selection=[('manual','Manual Source Doc.'),('oe_doc','OpenERP Source Doc.')], 
                    required=True
                    ),
                'origin_oe_doc_id' : fields.reference(string='Origin OE Doc.', 
                    selection=_selection_origin_oe_id, 
                    size=128
                    ), 
                'origin_doc' : fields.char(string='Origin Doc.', 
                    size=100
                    ),
                'function_origin_doc' : fields.function(string='Origin Doc.',
                    fnct=_function_origin_document, 
                    type='char', 
                    method=True, 
                    multi='document',
                    store=True
                    ),
                'origin_model_id' : fields.function(string='Origin Document Model',
                    fnct=_function_origin_document, 
                    type='many2one', 
                    method=True, 
                    multi='document',
                    relation='ir.model',
                    store=True
                    ),
                'origin_res_id' : fields.function(string='Origin ID',
                    fnct=_function_origin_document, 
                    type='integer', 
                    method=True, 
                    multi='document',
                    store=True
                    ),
    }

    _defaults = {
                'origin_type' : _default_origin_type,
                }


account_move()




